package com.nisum.consumer;

import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import com.nisum.entity.UserCardInfo;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.types.StructType;
import org.bson.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.nisum.utils.UserConstants.*;

public class SparkConsumer {
    static Logger log = LoggerFactory.getLogger(SparkConsumer.class);
    public static void main(String[] args) throws TimeoutException, StreamingQueryException {

        SparkSession spark = SparkSession.builder()
                .master("local[*]")
                .appName("MongoSparkConnectorIntro")
                .config("spark.mongodb.output.uri", USERDATA_HOST)
                .getOrCreate();
        log.debug("spark session is created");
        Map<String, String> kafkaConfigMap = new HashMap<>();
        kafkaConfigMap.put("kafka.bootstrap.servers", KAFKA_HOST);
        kafkaConfigMap.put("subscribe", TOPIC);
        kafkaConfigMap.put("startingOffsets", "earliest");
        kafkaConfigMap.put("failOnDataLoss", "false");
        kafkaConfigMap.put("kafka.con.group.id", "local");

        Dataset<Row> rawItemDF = spark
                .readStream()
                .format("kafka")
                .options(kafkaConfigMap)
                .load();
        log.debug("kafka is loaded");
        StructType structType = Encoders.bean(UserCardInfo.class).schema();

        Dataset<Row> df = rawItemDF
                .selectExpr("CAST(value as String) as message")
                .select(functions.from_json(functions.col("message"), structType).as("data"))
                .select("data.*")
                .repartition(400);
        log.debug("consumed data is type casting to string");
        Dataset<Row> filteredDataFrame = df.filter(functions.col("cardType").equalTo("Visa"));
        log.debug("data filtering is done with visa card type");

        Dataset<UserCardInfo> ds = filteredDataFrame.as(Encoders.bean(UserCardInfo.class));

        StreamingQuery query = ds.writeStream().outputMode(OutputMode.Append())
                .option("checkpointLocation", "checkpointlocaion/streamingjob")
                .foreach(new ForeachWriter<>() {
                    private MongoClient mongoClient;
                    private MongoDatabase database;
                    private MongoCollection<Document> collection;

                    @Override
                    public boolean open(long partitionId, long epochId) {
                        mongoClient = MongoClients.create(URI);
                        database = mongoClient.getDatabase(CARD_DETAILS_DB);
                        collection = database.getCollection(USER_CARD_DETAILS);
                        return true;
                    }

                    @Override
                    public void process(UserCardInfo userCardInfo) {
                        // Create a document with the object data
                        Document document = new Document();
                        document.append("userId", userCardInfo.getUserId());
                        document.append("userName", userCardInfo.getUserName());
                        document.append("regMobileNo", userCardInfo.getRegMobileNo());
                        document.append("gender", userCardInfo.getGender());
                        document.append("cardId", userCardInfo.getCardId());
                        document.append("cardType", userCardInfo.getCardType());
                        document.append("cardNumber", userCardInfo.getCardNumber());
                        document.append("cardStatus", userCardInfo.getCardStatus());

                        // Insert the document into the collection
                        collection.insertOne(document);
                        log.debug("document inserted" + document);
                    }

                    @Override
                    public void close(Throwable errorOrNull) {
                        // Close the MongoDB connection
                        mongoClient.close();
                    }
                }).start();
        query.awaitTermination();

    }
}
